import datetime

import pytest
from pydantic import ValidationError

from setra_client.models import (
    Batch,
    BatchErrors,
    CompleteBatch,
    ErrorBatch,
    Parameter,
    Transaction,
    Voucher,
)


def test_batch(batch_fixture):
    assert Batch(**batch_fixture)


def test_batch(batch_without_voucher_field):
    assert Batch(**batch_without_voucher_field)


def test_voucher(voucher_fixture):
    assert Voucher(**voucher_fixture)


def test_voucher_has_transactions(voucher_fixture):
    voucher = Voucher(**voucher_fixture)
    assert len(voucher.transactions) == 2
    assert voucher.transactions.__getitem__(0).amount == 1
    assert voucher.transactions.__getitem__(1).amount == 2


def test_transaction(trans_fixture):
    assert Transaction(**trans_fixture)


def test_transaction_fail(trans_fail_fixture):
    # Check missing required field fails
    with pytest.raises(ValidationError):
        Transaction(**trans_fail_fixture)


def test_batch_with_voucher(batch_with_voucher_fixture):
    assert Batch(**batch_with_voucher_fixture)


def test_batch_fail(batch_fail_fixture):
    # Using wrong data format, should fail:
    with pytest.raises(ValidationError):
        Batch(**batch_fail_fixture)


def test_error_batch(error_batch):
    eb = ErrorBatch(**error_batch)
    assert eb.id == 1


def test_batch_error(batch_errors):
    be = BatchErrors(**batch_errors)
    assert isinstance(be.batch_errors, list)
    assert isinstance(be.voucher_errors, list)
    assert isinstance(be.transaction_errors, list)


def test_batch_complete(complete_batch):
    bc = CompleteBatch(**complete_batch)
    assert bc.created == datetime.datetime(
        2020, 1, 2, 0, 0, tzinfo=datetime.timezone.utc
    )


def test_params(param_list):
    param = [Parameter(**i) for i in param_list]
    assert param
    assert param[0].created == datetime.datetime(
        2020, 1, 1, 2, 0, 0, tzinfo=datetime.timezone.utc
    )
